﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.Reflection;

namespace Components
{
    public partial class Form1 : Form
    {
        private MyObject data = new MyObject ();

        public Form1()
        {
            InitializeComponent();
            treeView1.ShowNodeToolTips = true;
            // display (treeView1.Nodes, this);
            propertyGrid1.SelectedObject = data;

            showInfo (data);
        }

        private void showInfo(object obj)
        {
            Type typ = obj.GetType ();
            // using System.Reflection;
            PropertyInfo [] prop = typ.GetProperties ();
            foreach (var p in prop)
            {
                String t = "";
                foreach (CustomAttributeData a in p.CustomAttributes)
                    t = t + a + " ";

                String[] info = new String [] 
                    { p.Name, 
                      p.PropertyType.Name,
                      "" + p.GetValue(obj),
                      t
                    };

                dataGridView1.Rows.Add(info);
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                var stream = new StreamReader (openFileDialog1.FileName);
                var reader = new XmlSerializer (typeof(MyObject));
                data = (MyObject) reader.Deserialize (stream);
                propertyGrid1.SelectedObject = data;
                stream.Close();
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                // using System.IO;
                StreamWriter stream =
                    new StreamWriter (saveFileDialog1.FileName);
                // using System.Xml.Serialization;
                XmlSerializer writer =
                    new XmlSerializer (typeof(MyObject));
                writer.Serialize (stream, data);

                stream.Close ();
            }
        }

        private void display (TreeNodeCollection branch, Control obj)
        {
            TreeNode node = new TreeNode ();
            node.Text = obj.Name + " : " + obj.GetType().Name;
            node.ToolTipText = obj.ToString ();
            node.Tag = obj;
            branch.Add (node);

            foreach (Control c in obj.Controls)
                display(node.Nodes, c);
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close ();
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            Control obj = e.Node.Tag as Control;
            // obj.BackColor = Color.LightGreen;
            propertyGrid1.SelectedObject = obj;
        }

    }
}
